/*******************************************************************************
 * exception.C - Exception base class
 *
 * $Id: exception.C,v 1.2 2005/04/26 11:12:31 mscheid Exp $
 *
 * Author: Matthias Scheidegger <mscheid@iam.unibe.ch>
 *
 * Copyright 2001, Matthias Scheidegger
 *
 ******************************************************************************/

#include "exception.H"
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>

static const char *critical_msg = "Critical failure, exception constructor failed! Bailing out";


Exception::Exception(const Exception &other)
{
    if (!other.msg) msg = 0;
    else msg = strdup(other.msg);
}



Exception& Exception::operator= (const Exception &other)
{
    if (&other == this) return *this;
    delete[] msg;
    if (!other.msg) msg = 0;
    else msg = strdup(other.msg);
}



Exception::~Exception()
{
    delete[] msg;
}



char* Exception::prepend_string(const char *pre, const char *str)
{
    int len = strlen(str);
    int prelen = strlen(pre);

    char *nmsg = new char[len + prelen + 3];
    if (!nmsg) mem_handler();
    
    sprintf(nmsg, "%s: %s", pre, str);
    return nmsg;
}



void Exception::set_string(const char *fmt, va_list vl)
{
    int res;
    char *new_fmt;

    new_fmt = prepend_string(class_name(), fmt);
    msg = new char[msg_len];
    if (!msg) mem_handler();

    res = vsnprintf(msg, msg_len, new_fmt, vl);

    if (res < 0) die(critical_msg);
    delete[] new_fmt;    
}



void Exception::set_string(int error_number, const char *func)
{
    char *tmpmsg;

    tmpmsg = strdup(strerror(error_number));
    if (!tmpmsg) mem_handler();

    msg = prepend_string(func, tmpmsg);
    delete[] tmpmsg;
    tmpmsg = msg;
    msg = prepend_string(class_name(), tmpmsg);
    delete[] tmpmsg;
}



void Exception::set_string(int error_number)
{
    char *tmpmsg;

    tmpmsg = strdup(strerror(error_number));
    if (!tmpmsg) mem_handler();

    msg = prepend_string(class_name(), tmpmsg);
    delete[] tmpmsg;
}



void Exception::set_string()
{
    char *tmpmsg;

    tmpmsg = strdup(standard_text());
    if (!tmpmsg) mem_handler();

    msg = prepend_string(class_name(), tmpmsg);
    delete[] tmpmsg;
}



void warn(const char *fmt, ...)
{
    va_list args;
    
    va_start(args, fmt);
    vfprintf(stderr, fmt, args);
    fprintf(stderr, "\n");
    fflush(stderr);
    va_end(args);    
}



void die(const char *fmt, ...)
{
    va_list args;
    
    va_start(args, fmt);
    vfprintf(stderr, fmt, args);
    fprintf(stderr, "\n");
    fflush(stderr);
    va_end(args);

    exit(2);
}


void mem_handler()
{
    die("Out of memory");
}
